#ifndef MODEL_HPP
#define MODEL_HPP

#include <cmath>
#include <iostream>

class comp_models {
    protected:
        int n;
        double * u0;
        double tau_birth;
        double tau_death;
    public:
        comp_models();
        comp_models(int n);
        comp_models(const comp_models & c);
        virtual double gives_R0();
        void getu0(double x, int i);
        void gettau(double tb, double td);
        virtual void transition_matrix_product(double * u, double * res);
};


class seir_model: public comp_models{
    protected:
        double gamma;
        double beta;
        double sigma;
    public:
        seir_model();
        seir_model(double beta, double gamma, double sigma);
        seir_model(const seir_model & s);
        double gives_R0();
        void transition_matrix_product(double * u, double * res);
};

class age_structured: public comp_models{
    protected:
        int nb_a;
        double age_max;
        double *ages;
        int index_var(int var,  int age){ return age +var*nb_a;}
    public:
        age_structured();
        age_structured(int nvar, int nb_a, double age_max);
        age_structured(const age_structured & s);
        double average_variables(int i);
        virtual double gives_R0();
        virtual void transition_matrix_product(double * u, double * res);
};

class sir_age_structured: public age_structured{
    protected:
        int nb_a;
        double age_max;
        double (*beta)(double a1, double a2);
        double (*gamma)(double a);
        double (*taub)(double a);
        double (*taud)(double a);
    public:
        sir_age_structured();
        sir_age_structured(int nb_a, double age_max);
        sir_age_structured(const sir_age_structured & s);
        double gives_R0();
        void getbeta(double (*beta)(double a1, double a2));
        void getgamma(double (*gamma)(double a1));
        void gettb(double (*tb)(double a1));
        void gettd(double (*td)(double a1));
        void transition_matrix_product(double * u, double * res);
};

#endif
