#include <cmath>
#include <iostream>
using namespace std;

#include "comp_models.hpp"



comp_models::comp_models(){
     n = 0;
     u0 = NULL;
     tau_death = 0.0;
     tau_birth =0.0;
 };
        
comp_models::comp_models(int m){
    n= m;
    u0 = new double[n];
    tau_death = 0.0;
    tau_birth = 0.0;
}

comp_models::comp_models(const comp_models & c){
    n = c.n;
    tau_death = c.tau_death;
    tau_birth = c.tau_birth;
    if(c.u0 != NULL){
        u0 = new double[n];
        for(int i=0;i<n;i++){
            u0[i]=c.u0[i];
        }
    }
}

double comp_models::gives_R0(){
    cout<< "Pas de R0 "<<endl;
    return 0;
}
void comp_models::getu0(double x, int i){
    u0[i]=x;
}

void comp_models:: transition_matrix_product(double * u, double * res){
    cout<< "Pas de transition "<<endl;
}
void comp_models:: gettau(double tb, double td){
    tau_birth = tb;
    tau_death = td;
}

////////////////////////////////////////////////////

seir_model::seir_model(): comp_models(4){}

seir_model::seir_model(double be, double ga, double sig): comp_models(4){
    beta = be;
    gamma = ga;
    sigma = sig;
}   
    
seir_model::seir_model(const seir_model & s): comp_models(s){
    gamma = s.gamma;
    beta = s.beta;
    sigma = s.sigma;
}  

double seir_model::gives_R0(){
    return sigma*beta*tau_birth/((tau_death+sigma)*(gamma+tau_death)*tau_death);
}

void seir_model::transition_matrix_product(double * u, double * res){
    double S = u[0]; 
    double E=  u[1];
    double I = u[2];
    double R = u[3];

    res[0] = - beta*S*I + tau_birth*(S+E+I+R) - tau_death*S;
    res[1] = beta*S*I -sigma*E - tau_death*E;
    res[2] = sigma*E-gamma*I -tau_death*I;
    res[3] = gamma*I - tau_death*R;
}

////////////////////////////////////////////////////


age_structured:: age_structured(): comp_models(){};

age_structured:: age_structured(int nvar, int nba, double am): comp_models(nvar*nba){
    nb_a = nba;
    age_max = am;
    ages = new double[nba];
    for(int j=0;j<nb_a;j++){
        ages[j]= j*age_max/nb_a;
    }
};

age_structured::age_structured(const age_structured & a): comp_models(a){
    nb_a = a.nb_a;
    age_max = a.age_max;
};

double age_structured::gives_R0(){
    cout<< "rien "<<endl;
    return 0;
}
double age_structured::average_variables(int i){
    double res=0.0;
    for(int j=0;j<nb_a;j++){
        res = res + u0[index_var(i,j)];
    }
    res = res/age_max;
    return res;
}

void age_structured::transition_matrix_product(double * u, double * res){
    cout<< "rien "<<endl;
}


////////////////////////////////////////////////////

sir_age_structured:: sir_age_structured(): age_structured(){};

sir_age_structured:: sir_age_structured(int nba, double am): age_structured(3,nba,am){
    nb_a = nba;
    age_max = am;
}

sir_age_structured::sir_age_structured(const sir_age_structured & s): age_structured(s){}

double sir_age_structured::gives_R0(){
    cout<< " trop compliqué "<<endl;
    return 0;
}


void sir_age_structured::transition_matrix_product(double * u, double * res){
    double lambda_a=0.0;
    double N=0.0;
    for(int i=0;i<nb_a;i++){
        for(int j=0;j<nb_a;j++){
            lambda_a =lambda_a + beta(ages[i],ages[j])*u[index_var(1,j)];
        }
        N = u[index_var(0,i)] +  u[index_var(1,i)] +  u[index_var(2,i)];
        res[0] = - lambda_a*u[index_var(0,i)] +tau_birth*taub(ages[i])*N - tau_death*taud(ages[i])*u[index_var(0,i)];
        res[1] = lambda_a*u[index_var(0,i)] - gamma(ages[i])*u[index_var(1,i)] - tau_death*taud(ages[i])*u[index_var(1,i)];
        res[2] = gamma(ages[i])*u[index_var(1,i)] - tau_death*taud(ages[i])*u[index_var(2,i)];
    }
}

void sir_age_structured::getbeta(double (*b)(double a1, double a2)){
    beta = b;
}
void sir_age_structured::getgamma(double (*g)(double a1)){
    gamma = g;
}
void sir_age_structured::gettb(double (*tb)(double a1)){
    taub = tb;
}
void sir_age_structured::gettd(double (*td)(double a1)){
    taud= td;
}